#pragma once

#include "dbus.hpp"

// Implementation of the `Printer` interface which sends its output to a
// file descriptor.
class PrinterFD final : public Printer {
  // This class is not responsible for closing the file descriptor.
  const int fd_;

  // Decimal or hexadecimal output?
  const int base_;

  const size_t tabsize_;

  void printBytes(const char* buf, size_t bufsize);

public:
  PrinterFD(int fd, size_t base, size_t tabsize) :
    fd_(fd), base_(base), tabsize_(tabsize)
  {}

  void printChar(char c);
  void printUint8(uint8_t x);
  void printUint32(uint32_t x);
  void printInt32(int32_t x);
  void printUint64(uint64_t x);
  void printInt64(int64_t x);
  void printString(const std::string& str);

  // Print a newline character, followed by `tabsize_ * indent`
  // space chracters.
  void printNewline(size_t indent);
};
