#pragma once

#include <string>
#include <vector>

#define likely(x)       __builtin_expect(!!(x), 1)
#define unlikely(x)     __builtin_expect(!!(x), 0)

// This class automatically closes the file descriptor in its destructor.
class AutoCloseFD {
  const int fd_;

public:
  AutoCloseFD() = delete;  // No default constructor.
  explicit AutoCloseFD(const int fd) : fd_(fd) {}
  ~AutoCloseFD();

  int get() const { return fd_; }
};

// Shorthand for creating a std::string from a string literal.
inline std::string _s(const char* s) {
  return std::string(s);
}

// Utility for constructing a std::vector.
template <class... Ts>
std::vector<typename std::common_type<Ts...>::type> _vec(Ts&&... args) {
  std::vector<typename std::common_type<Ts...>::type> result;
  result.reserve(sizeof...(args));
  int bogus[] = { ((void)result.emplace_back(std::forward<Ts>(args)), 0)... };
  static_assert(sizeof(bogus) == sizeof(int) * sizeof...(args));
  return result;
}

// Get the process's start time by reading /proc/[pid]/stat.
uint64_t process_start_time(pid_t pid);
